﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;
using System.ComponentModel.Design;
using Microsoft.Win32;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;
using EnvDTE80;
using EnvDTE;
using System.Windows.Documents;
using System.Collections.Generic;
using AssemblyInfoEditor.Project;

namespace AssemblyInfoEditor
{
    /// <summary>
    /// This is the class that implements the package exposed by this assembly.
    ///
    /// The minimum requirement for a class to be considered a valid package for Visual Studio
    /// is to implement the IVsPackage interface and register itself with the shell.
    /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
    /// to do it: it derives from the Package class that provides the implementation of the 
    /// IVsPackage interface and uses the registration attributes defined in the framework to 
    /// register itself and its components with the shell.
    /// </summary>
    // This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is
    // a package.
    [PackageRegistration(UseManagedResourcesOnly = true)]
    // This attribute is used to register the informations needed to show the this package
    // in the Help/About dialog of Visual Studio.
    [InstalledProductRegistration("#110", "#112", "1.0", IconResourceID = 400)]
    // This attribute is needed to let the shell know that this package exposes some menus.
    [ProvideMenuResource("Menus.ctmenu", 1)]
    // This attribute registers a tool window exposed by this package.
    [ProvideToolWindow(typeof(AssemblyInfoEditorToolWindow))]
    [Guid(GuidList.guidVersionPackagePkgString)]
    public sealed class AssemblyInfoEditorPackage : Package
    {
        private static EnvDTE.DTE ideAppObject;
        private DTE ide;
        private SolutionEvents ideSolutionEvents;
        private DTEEvents ideDteEvents;
        private static List<ProjectAssemblyInfo> projects;

        /// <summary>
        /// Default constructor of the package.
        /// Inside this method you can place any initialization code that does not require 
        /// any Visual Studio service because at this point the package object is created but 
        /// not sited yet inside Visual Studio environment. The place to do all the other 
        /// initialization is the Initialize method.
        /// </summary>
        public AssemblyInfoEditorPackage()
        {
            Trace.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering constructor for: {0}", this.ToString()));
        }

        /// <summary>
        /// This function is called when the user clicks the menu item that shows the 
        /// tool window. See the Initialize method to see how the menu item is associated to 
        /// this function using the OleMenuCommandService service and the MenuCommand class.
        /// </summary>
        private void ShowToolWindow(object sender, EventArgs e)
        {
            // Get the instance number 0 of this tool window. This window is single instance so this instance
            // is actually the only one.
            // The last flag is set to true so that if the tool window does not exists it will be created.
            ToolWindowPane window = this.FindToolWindow(typeof(AssemblyInfoEditorToolWindow), 0, true);
 
            if ((null == window) || (null == window.Frame))
            {
                throw new NotSupportedException(Resources.CanNotCreateWindow);
            }

            IVsWindowFrame windowFrame = (IVsWindowFrame)window.Frame;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(windowFrame.Show());
        }

        /////////////////////////////////////////////////////////////////////////////
        // Overriden Package Implementation
        #region Package Members

        /// <summary>
        /// Initialization of the package; this method is called right after the package is sited, so this is the place
        /// where you can put all the initilaization code that rely on services provided by VisualStudio.
        /// </summary>
        protected override void Initialize()
        {
            Trace.WriteLine (string.Format(CultureInfo.CurrentCulture, "Entering Initialize() of: {0}", this.ToString()));
            base.Initialize();

            ide = (DTE)GetService(typeof(DTE));
            ideSolutionEvents = ide.Events.SolutionEvents;
            ideDteEvents = ide.Events.DTEEvents;

            ideSolutionEvents.Opened += new _dispSolutionEvents_OpenedEventHandler(ideSolutionEvents_Opened);
            ideSolutionEvents.BeforeClosing += new _dispSolutionEvents_BeforeClosingEventHandler(ideSolutionEvents_BeforeClosing);

            ideDteEvents.OnBeginShutdown += new _dispDTEEvents_OnBeginShutdownEventHandler(ideDteEvents_OnBeginShutdown);

            prjManager = new Project.ProjectManager(ide);

            // Add our command handlers for menu (commands must exist in the .vsct file)
            OleMenuCommandService mcs = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if ( null != mcs )
            {
                // Create the command for the menu item.
                CommandID menuCommandID = new CommandID(GuidList.guidVersionPackageCmdSet, (int)PkgCmdIDList.cmdidVersionSettings);
                MenuCommand menuItem = new MenuCommand(MenuItemCallback, menuCommandID );
                mcs.AddCommand( menuItem );

                // Create the command for the tool window
                CommandID toolwndCommandID = new CommandID(GuidList.guidVersionPackageCmdSet, (int)PkgCmdIDList.cmdidVersionSettingsWindow);
                MenuCommand menuToolWin = new MenuCommand(ShowToolWindow, toolwndCommandID);
                
                mcs.AddCommand( menuToolWin );
            }

            projects = new List<ProjectAssemblyInfo>();
        }

        #endregion

        /// <summary>
        /// This function is the callback used to execute a command when the a menu item is clicked.
        /// See the Initialize method to see how the menu item is associated to this function using
        /// the OleMenuCommandService service and the MenuCommand class.
        /// </summary>
        private void MenuItemCallback(object sender, EventArgs e)
        {
            ShowToolWindow(this, new EventArgs());
        }

        #region IDE Event Handlers

        void ideSolutionEvents_Opened()
        {
            // reload the list of projects
            //ReloadProjects();

            //ProjectManager.LoadProjectInfo();
        }

        void ideSolutionEvents_BeforeClosing()
        {
            CloseToolWindow();
        }

        void ideDteEvents_OnBeginShutdown()
        {
            CloseToolWindow();
        }

        #endregion

        private void CloseToolWindow()
        {
            ToolWindowPane window = this.FindToolWindow(typeof(AssemblyInfoEditorToolWindow), 0, false);

            if ((window != null) && (window.Frame != null))
            {
                IVsWindowFrame windowFrame = (IVsWindowFrame)window.Frame;
                windowFrame.CloseFrame((uint)Microsoft.VisualStudio.Shell.Interop.__FRAMECLOSE.FRAMECLOSE_NoSave);
            }
        }

        public static DTE IDEAppObject
        {
            get
            {
                if (ideAppObject != null)
                {
                    return ideAppObject;
                }
                else
                {
                    ideAppObject = (DTE)GetGlobalService(typeof(DTE));
                    return ideAppObject;
                }
            }
            set
            {
                if (ideAppObject != null)
                {
                    ideAppObject = value;
                }
                else
                {
                    ideAppObject = (DTE)GetGlobalService(typeof(DTE));
                    ideAppObject = value;
                }
            }
        }

        private static Project.ProjectManager prjManager;

        public static Project.ProjectManager ProjectManager
        {
            get { return prjManager; }
            set { prjManager = value; }
        }

        public static Project.ProjectManager CreateProjectManager()
        {
            if (ideAppObject == null)
            {
                ideAppObject = (DTE)GetGlobalService(typeof(DTE));
            }

            return new ProjectManager(ideAppObject);
        }
    }
}
